function [olstablecol, wlstablecol] = make_intlvrp_col(in,params)
% function tablecol = make_intlvrp_col(in,params)

% number of lags for NW?
params.nlags = ceil(1.5*params.nfutperiods); 

% do we use hodrick (1992) trick to undo overlap?
if( params.usehod )  
    % do we simulate standard errors or use asymptotic?
    if( params.simse )
        [ols_stats, wls_stats] = olswlsbootstrap_keepxsig(in,params); 
        olstablecol = [ ols_stats.data_coeff(2);  ...
            abs(ols_stats.data_coeff(2)/ols_stats.std_he_sim_coeff(2)); 100*ols_stats.R2 ]; 

        wlstablecol = [ wls_stats.data_coeff(2); ...
            abs(wls_stats.data_coeff(2)/wls_stats.std_he_sim_coeff(2)); 100*wls_stats.R2 ]; 
    else
        params.nsim = 10; 
        [ols_stats, wls_stats, ~ ] = olswlsbootstrap_keepxsig(in,params); 
        olstablecol = [ ols_stats.data_coeff(2);  ...
            abs(ols_stats.data_coeff(2)/ols_stats.data_se_nw(2)); 100*ols_stats.R2 ]; 

        wlstablecol = [ wls_stats.data_coeff(2); ...
            abs(wls_stats.data_coeff(2)/wls_stats.data_se_nw(2)); 100*wls_stats.R2 ]; 
    end
else
    % if we are not using hodrick (1992) trick to undo overlap, cannot do
    % simulated standard errors
    csumr = cumsum(in.r); 
    data_lhs = params.lhsmultiplier* ...
       (csumr(1+params.nfutperiods:end,1) - csumr(1:end-params.nfutperiods));  
    
    data_rhs = [ ones(length(in.r)-params.nfutperiods,1) in.x(1:end-params.nfutperiods) ]; 
    
    nlags = ceil(params.nfutperiods*1.5); 
    
    [ ols_coeff, ols_se_nw, ~, R2adj_ols ] = olsgmm(data_lhs,data_rhs,nlags,1); 
    [ wls_coeff, wls_se_nw, ~, R2adj_wls ] = wlsev(data_lhs,data_rhs,in.predvar(1:end-params.nfutperiods),nlags,1); 
    
    olstablecol = [ols_coeff(2,1); abs(ols_coeff(2,1)/ols_se_nw(2,1)); 100*R2adj_ols]; 
    wlstablecol = [wls_coeff(2,1); abs(wls_coeff(2,1)/wls_se_nw(2,1)); 100*R2adj_wls]; 
end