close all; 

nfutperiods = 21; 
maxcolor = 0.85; 
pos = [100 100 750 350] ; % FOR PAPER
pres_pos = [100 100 750 500] ; % FOR PRES
scatterselector = 1:5:height(vixsample_d)-nfutperiods; 

T = height(vixsample_d); 

% use 21 lags for NW
nlags = 21; 

% Redo x so we are estimating the t+1,t+nfutperiods overlapping
% predicability
x = vixsample_d.vrp_dy; 
rollx = conv(x,ones(1,nfutperiods),'valid'); 
rollx = rollx(~isnan(rollx),1); 
rollx = rollx*var(x)/var(rollx);

rhs_reg = [ ones(T-nfutperiods,1) rollx(1:end-1,1) ]; 

% estimate actual regressions
[ olsb, olsseb ] = olsgmm(1200*log(1+vixsample_d.mktrf(1+nfutperiods:end,1)),rhs_reg,nlags,1); 
[ wlsevb, wlsevseb ] = wlsev(1200*log(1+vixsample_d.mktrf(1+nfutperiods:end,1)),rhs_reg,vixsample_d.sigsq_both(nfutperiods:end-1,1),nlags,1); 

% Now make figure
cslr = cumsum(vixsample_d.mktrf); 
vixsample_d.next21 = 1200*[exp(cslr(nfutperiods+1:end,1)-cslr(1:end-nfutperiods,1))-1; NaN(nfutperiods,1)]; 
x_fig = x(1:end-nfutperiods,1); 
lhs_fig = vixsample_d.next21(1:end-nfutperiods,1); 
sigsq_fig = vixsample_d.sigsq_both(1:end-nfutperiods,1);

maxx = 1.05*max(x_fig); 
minx = 1.05*min(x_fig); 

% Create figure
figure1 = figure;

% Create axes
axes1 = axes('Parent',figure1,'FontName','CMU Serif');
hold on;
box on; 

% plot fitted lines 
plot([minx; maxx],[1 minx ; 1 maxx]*olsb,'color',[0 0 0],'LineStyle','-.'); 
plot([minx; maxx],[1 minx ; 1 maxx]*wlsevb,'color',[0 0 0]); 
legendstrols = sprintf('b_{OLS} = %.3f, SE = %.3f',olsb(2,1),olsseb(2,1)); 
legendstrwls = sprintf('b_{WLS-EV} = %.3f, SE = %.3f',wlsevb(2,1),wlsevseb(2,1)); 
legend(legendstrols,legendstrwls,'location','NorthWest'); 
hold on;

% scatterplot points
scatterlhs = lhs_fig(scatterselector,1); 
scatterx = x_fig(scatterselector,:); 
scattersigsq = sigsq_fig(scatterselector,1); 
weights = 1./sqrt(scattersigsq); 
colors = maxcolor*(ones(length(scatterlhs),3)-weights*[1 1 1]/max(weights));
scatter(scatterx,scatterlhs,3,colors,'o','filled','HandleVisibility','off'); 

% label axis
xlim([minx maxx]); 
xlabel('DY VRP_d (Monthly %)^2'); 
ylabel('r_{d+1,d+21} (Annualized % log)'); 

set(gcf,'Position',pos); 
set(gcf,'Color',[1 1 1]); 

% export
export_fig(gcf,[ base_folder 'dyplot.png' ],'-r600'); 

set(gcf,'Position',pres_pos); 

export_fig(gcf,[ base_folder 'dyplot_pres.png' ],'-r600'); 

close all; 

% Redo x so we are estimating the t+1,t+nfutperiods overlapping
% predicability
x = vixsample_d.vrp_btz; 
rollx = conv(x,ones(1,nfutperiods),'valid'); 
rollx = rollx(~isnan(rollx),1); 
rollx = rollx*var(x)/var(rollx);

rhs_reg = [ ones(T-nfutperiods,1) rollx(1:end-1,1) ]; 

% estimate actual regressions
[ olsb, olsseb ] = olsgmm(1200*log(1+vixsample_d.mktrf(1+nfutperiods:end,1)),rhs_reg,nlags,1); 
[ wlsevb, wlsevseb ] = wlsev(1200*log(1+vixsample_d.mktrf(1+nfutperiods:end,1)),rhs_reg,vixsample_d.sigsq_both(nfutperiods:end-1,1),nlags,1); 

% Now make figure
cslr = cumsum(vixsample_d.mktrf); 
vixsample_d.next21 = 1200*[exp(cslr(nfutperiods+1:end,1)-cslr(1:end-nfutperiods,1))-1; NaN(nfutperiods,1)]; 
x_fig = x(1:end-nfutperiods,1); 
lhs_fig = vixsample_d.next21(1:end-nfutperiods,1); 
sigsq_fig = vixsample_d.sigsq_both(1:end-nfutperiods,1);

maxx = 1.05*max(x_fig); 
minx = 1.05*min(x_fig); 

% Create figure
figure1 = figure;

% Create axes
axes1 = axes('Parent',figure1,'FontName','CMU Serif');
hold on;
box on; 

% plot fitted lines 
plot([minx; maxx],[1 minx ; 1 maxx]*olsb,'color',[0 0 0],'LineStyle','-.'); 
plot([minx; maxx],[1 minx ; 1 maxx]*wlsevb,'color',[0 0 0]); 
legendstrols = sprintf('b_{OLS} = %.3f, SE = %.3f',olsb(2,1),olsseb(2,1)); 
legendstrwls = sprintf('b_{WLS-EV} = %.3f, SE = %.3f',wlsevb(2,1),wlsevseb(2,1)); 
legend(legendstrols,legendstrwls,'location','NorthWest'); 
hold on;

% scatterplot points
scatterlhs = lhs_fig(scatterselector,1); 
scatterx = x_fig(scatterselector,:); 
scattersigsq = sigsq_fig(scatterselector,1); 
weights = 1./sqrt(scattersigsq); 
colors = maxcolor*(ones(length(scatterlhs),3)-weights*[1 1 1]/max(weights));
scatter(scatterx,scatterlhs,3,colors,'o','filled','HandleVisibility','off');  

% label axis
xlim([minx maxx]); 
xlabel('BTZ VRP_d (Monthly %)^2'); 
ylabel('r_{d+1,d+21} (Annualized % log)'); 

set(gcf,'Position',pos); 
set(gcf,'Color',[1 1 1]); 

% export
export_fig(gcf,[ base_folder 'btzplot.png' ],'-r600');

set(gcf,'Position',pres_pos); 

export_fig(gcf,[ base_folder 'btzplot_pres.png' ],'-r600');
