function [ isrsq, oosrsq ] = oosisrsq(r,r_sim,x,sigsq,realvar,params)
% function [ isrsq, oosrsq ] = oosisrsq(r,simr,x,sigsq,realvar,params)
% r is an index return, x is a potentially multivariate predictor (no
% constant), sigsq is the ex-ante variance measure as of t (predicting t+1),
% realvar is the realized variance during period t
%
% Computes isrsq = in-sample R^2 
%          oosrsq = out-of-sample R^2 using the Goyal Welch rolling
%          approach

T = length(r)-params.nfutperiods; 
K = size(x,2); 
nsim = size(r_sim,2); 

% Redo x so we are estimating the t+1,t+nfutperiods overlapping
% predicability
rhs_reg = makerhs(x,params.nfutperiods); 
lhs_reg = r(1+params.nfutperiods:end,1); 
% lhs_sim = r_sim(1+params.nfutperiods:end,:); 

rhs_rsq = [ones(T,1) x(1:end-params.nfutperiods,1)]; 
csr = cumsum(r); 
lhs_rsq = csr(1+params.nfutperiods:end) - csr(1:end-params.nfutperiods); 
csr_sim = cumsum(r_sim); 
lhs_rsq_sim = csr_sim(1+params.nfutperiods:end,:) - csr_sim(1:end-params.nfutperiods,:); 

% IS rsqs
% ols
olscoeff = olsgmm(lhs_reg,rhs_reg,0,-1); 
olscoeff_rsq(2,1) = olscoeff(2,1)*var(rhs_reg(:,2))/var(rhs_rsq(:,2)); 
olscoeff_rsq(1,1) = mean(lhs_reg)*params.nfutperiods - olscoeff_rsq(2,1)*mean(rhs_rsq(:,2)); 
olseps = lhs_rsq-rhs_rsq*olscoeff_rsq;
s2 = mean(olseps.^2);
olsrsq = (1-s2/var(lhs_rsq)); 
olsrsqadj = olsrsq*(T-1)/(T-K-1) - K / (T-K-1);

isrsq.ols = olsrsqadj;

% wls
wlscoeff = wlsev(lhs_reg,rhs_reg,sigsq(params.nfutperiods:end-1,1),0,-1); 
wlscoeff_rsq(2,1) = wlscoeff(2,1)*var(rhs_reg(:,2))/var(rhs_rsq(:,2)); 
wls_mean = wlsev(lhs_reg,rhs_reg(:,1),sigsq(params.nfutperiods:end-1,1),0,-1); 
wlscoeff_rsq(1,1) = wls_mean*params.nfutperiods - wlscoeff_rsq(2,1)*mean(rhs_rsq(:,2)); 
wlseps = lhs_rsq-rhs_rsq*wlscoeff_rsq;
s2 = mean(wlseps.^2);
wlsrsq = (1-s2/var(lhs_rsq)); 
wlsrsqadj = wlsrsq*(T-1)/(T-K-1) - K / (T-K-1);

isrsq.wls = wlsrsqadj;

ols_coeff = zeros(length(r),1); 
ols_predy = zeros(length(r),1); 
ols_ct_predy = zeros(length(r),1); 
ols_ptv_coeff = zeros(length(r),1); 
ols_ptv_predy = zeros(length(r),1); 

ols_coeff_sim = zeros(length(r),nsim); 
ols_predy_sim = zeros(length(r),nsim); 
ols_ct_predy_sim = zeros(length(r),nsim); 
ols_ptv_predy_sim = zeros(length(r),nsim); 

wls_coeff = zeros(length(r),1); 
wls_predy = zeros(length(r),1); 
wls_ct_predy = zeros(length(r),1);
wls_ptv_coeff = zeros(length(r),1);
wls_ptv_predy = zeros(length(r),1);

wls_coeff_sim = zeros(length(r),nsim); 
wls_predy_sim = zeros(length(r),nsim); 
wls_ct_predy_sim = zeros(length(r),nsim); 
wls_ptv_predy_sim = zeros(length(r),nsim); 

simple_avg = zeros(length(r),1); 
simple_avg_wls = zeros(length(r),1); 
simple_avg_sim = zeros(length(r),nsim); 
simple_avg_sim_wls = zeros(length(r),nsim); 

fprintf('%3d%%',0);
% OOS
for j = params.ntraining+1:length(r)
%     tic;
    % Only allowed to use past data for this rolling approach,
    % so cut everything down
    pastrhs = makerhs(x(1:j,1),params.nfutperiods); 
    pastlhs = r(1+params.nfutperiods:j,1); 
    pastlhs_sim = r_sim(1+params.nfutperiods:j,:);
    
    pastrhs_rsq = rhs_rsq(1:j-params.nfutperiods,:); 
    
    pastsigsq = sigsq(params.nfutperiods:j-1,:); 
    pastrealvar = realvar(1+params.nfutperiods:j,:); 
   
      % benchmark in GW is using simple average of past returns
%     simple_avg(j,1) = mean(pastlhs_rsq);  
    simple_avg(j,1) = params.nfutperiods*mean(pastlhs);  
    simple_avg_wls(j,1) = params.nfutperiods*wlsev(pastlhs,ones(length(pastlhs),1),pastsigsq,0,-1,pastrealvar); 
    simple_avg_sim(j,:) = params.nfutperiods*mean(pastlhs_sim,1);  
    simple_avg_sim_wls(j,:) = params.nfutperiods*wlsev(pastlhs_sim,ones(size(pastlhs,1),1),pastsigsq,0,-1,pastrealvar); 
    
    % ols, actual returns
    ols_bv = pastrhs \ pastlhs; 
    ols_bv_rsq(2,1) = ols_bv(2,1)*var(pastrhs(:,2))/var(pastrhs_rsq(:,2)); 
    ols_bv_rsq(1,1) = simple_avg(j,1) - ols_bv_rsq(2,1)*mean(pastrhs_rsq(:,2)); 
    ols_coeff(j,1) = ols_bv_rsq(2,1);
    ols_predy(j,1) = [1 x(j,:)]*ols_bv_rsq;

    % ols, sim returns
    ols_bv_sim = pastrhs \ pastlhs_sim; 
    ols_bv_sim_rsq(2,:) = ols_bv_sim(2,:)*var(pastrhs(:,2))/var(pastrhs_rsq(:,2)); 
    ols_bv_sim_rsq(1,:) = simple_avg_sim(j,:) - ols_bv_sim_rsq(2,:)*mean(pastrhs_rsq(:,2)); 
    ols_coeff_sim(j,:) = ols_bv_sim_rsq(2,:);
    ols_predy_sim(j,:) = [1 x(j,:)]*ols_bv_sim_rsq;
    
    % Since we include both past sigsq and pastrealvar, wlsev function does
    % first-stage variance prediction regression using only past data
    [wls_bv, ~, ~, ~, ~, wlsev_firststage] = wlsev(pastlhs,pastrhs,pastsigsq,0,-1,pastrealvar); 
    wls_bv_rsq(2,1) = wls_bv(2,1)*var(pastrhs(:,2))/var(pastrhs_rsq(:,2)); 
    wls_bv_rsq(1,1) = simple_avg_wls(j,1) - wls_bv_rsq(2,1)*mean(pastrhs_rsq(:,2)); 
    wls_coeff(j,1) = wls_bv_rsq(2,1);
    wls_predy(j,1) = [1 x(j,:)]*wls_bv_rsq; 
    
    %wls, sim returns
    wls_bv_sim = wlsev(pastlhs_sim,pastrhs,pastsigsq,0,-1,pastrealvar); 
    wls_bv_sim_rsq(2,:) = wls_bv_sim(2,:)*var(pastrhs(:,2))/var(pastrhs_rsq(:,2)); 
    wls_bv_sim_rsq(1,:) = simple_avg_sim_wls(j,:) - wls_bv_sim_rsq(2,:)*mean(pastrhs_rsq(:,2)); 
    wls_coeff_sim(j,:) = wls_bv_sim_rsq(2,:);
    wls_predy_sim(j,:) = [1 x(j,:)]*wls_bv_sim_rsq;

    % make Campbell-Thompson adjustments
    if( sign(ols_coeff(j,1)) ~= params.ct_sign )
        ols_ct_predy(j,1) = simple_avg(j,1);
    elseif(  ols_predy(j,1) < 0 )
        ols_ct_predy(j,1) = 0; 
    else 
        ols_ct_predy(j,1) = ols_predy(j,1);
    end

%     % start with standard predy
    ols_ct_predy_sim(j,:) = ols_predy_sim(j,:); 
    
    % modify if sign of coeff is wrong
    sign_wrong = sign(ols_coeff_sim(j,:)) ~= params.ct_sign; 
    ols_ct_predy_sim(j,sign_wrong) = simple_avg_sim(j,sign_wrong); 
    
    % or predicted excess return is negative
    pred_neg = ols_predy_sim(j,:) < 0 ; 
    ols_ct_predy_sim(j,pred_neg) = 0; 

    if( sign(wls_coeff(j,1)) ~= params.ct_sign )
        wls_ct_predy(j,1) = simple_avg_wls(j,1);
    elseif(  wls_predy(j,1) < 0 )
        wls_ct_predy(j,1) = 0; 
    else
        wls_ct_predy(j,1) = wls_predy(j,1);
    end
    
    % now CT for sims
    % start with standard predy
    wls_ct_predy_sim(j,:) = wls_predy_sim(j,:); 
    
    % modify if sign of coeff is wrong
    sign_wrong = sign(wls_coeff_sim(j,:)) ~= params.ct_sign; 
    wls_ct_predy_sim(j,sign_wrong) = simple_avg_sim_wls(j,sign_wrong); 
    
    % or predicted excess return is negative
    pred_neg = wls_predy_sim(j,:) < 0 ; 
    wls_ct_predy_sim(j,pred_neg) = 0; 

    % compute ptv coeff & predy using OLS
    ols_bv_ptv = wls_ptv(pastlhs,pastrhs,ones(length(pastlhs),1),wlsev_firststage.predvar,params.ptv_minsr,params.ptv_maxsr); 
    ols_bv_ptv_rsq(2,1) = ols_bv_ptv(2,1)*var(pastrhs(:,2))/var(pastrhs_rsq(:,2)); 
    ols_bv_ptv_rsq(1,1) = simple_avg(j,1) - ols_bv_ptv_rsq(2,1)*mean(pastrhs_rsq(:,2)); 
    ols_ptv_coeff(j,1) = ols_bv_ptv_rsq(2,1);
    ols_ptv_predy(j,1) = [1 x(j,:)]*ols_bv_ptv_rsq;
    
    % compute ptv coeff & predy using OLS
    % for this, need wls-ev weights!
    wls_bv_ptv = wls_ptv(pastlhs,pastrhs,1./wlsev_firststage.predvar,wlsev_firststage.predvar,params.ptv_minsr,params.ptv_maxsr); 
    wls_bv_ptv_rsq(2,1) = wls_bv_ptv(2,1)*var(pastrhs(:,2))/var(pastrhs_rsq(:,2)); 
    wls_bv_ptv_rsq(1,1) = simple_avg_wls(j,1) - wls_bv_ptv_rsq(2,1)*mean(pastrhs_rsq(:,2)); 
    wls_ptv_coeff(j,1) = wls_bv_ptv_rsq(2,1);
    wls_ptv_predy(j,1) = [1 x(j,:)]*wls_bv_ptv_rsq;
    
    % would have to loop through all sims and re-do the PTV constrained
    % estimation, making this take like 24 hours PER PREDICTOR
    % instead, going to just copy the CT values for simplicity
    ols_ptv_predy_sim(j,:) = ols_ct_predy_sim(j,:); 
    wls_ptv_predy_sim(j,:) = wls_ct_predy_sim(j,:); 

    % update display
    fprintf('\b\b\b\b%3d%%',floor(100*(j-params.ntraining+1)/(length(r)-params.ntraining+1)));
end
fprintf('\b\b\b\bDone\n'); 
% end

ols_coeff = ols_coeff(params.ntraining+1:end,:);
ols_predy = ols_predy(params.ntraining+1:end-params.nfutperiods,:);
wls_coeff = wls_coeff(params.ntraining+1:end,:);
wls_predy = wls_predy(params.ntraining+1:end-params.nfutperiods,:);

ols_predy_sim = ols_predy_sim(params.ntraining+1:end-params.nfutperiods,:);
ols_ct_predy_sim = ols_ct_predy_sim(params.ntraining+1:end-params.nfutperiods,:);
ols_ptv_predy_sim = ols_ptv_predy_sim(params.ntraining+1:end-params.nfutperiods,:);
wls_predy_sim = wls_predy_sim(params.ntraining+1:end-params.nfutperiods,:);
wls_ct_predy_sim = wls_ct_predy_sim(params.ntraining+1:end-params.nfutperiods,:);
wls_ptv_predy_sim = wls_ptv_predy_sim(params.ntraining+1:end-params.nfutperiods,:);

ols_ct_predy = ols_ct_predy(params.ntraining+1:end-params.nfutperiods,:);
wls_ct_predy = wls_ct_predy(params.ntraining+1:end-params.nfutperiods,:);

ols_ptv_coeff = ols_ptv_coeff(params.ntraining+1:end,:); 
ols_ptv_predy = ols_ptv_predy(params.ntraining+1:end-params.nfutperiods,:); 
wls_ptv_coeff = wls_ptv_coeff(params.ntraining+1:end,:); 
wls_ptv_predy = wls_ptv_predy(params.ntraining+1:end-params.nfutperiods,:); 

simple_avg = simple_avg(params.ntraining+1:end-params.nfutperiods,1);
simple_avg_wls = simple_avg_wls(params.ntraining+1:end-params.nfutperiods,1);
simple_avg_sim = simple_avg_sim(params.ntraining+1:end-params.nfutperiods,:);
simple_avg_sim_wls = simple_avg_sim_wls(params.ntraining+1:end-params.nfutperiods,:);

% % output rsqs
oosrsq.ols = computeoosrsq(lhs_rsq(params.ntraining+1:end,1),ols_predy,simple_avg); 
oosrsq.ols_ct = computeoosrsq(lhs_rsq(params.ntraining+1:end,1),ols_ct_predy,simple_avg); 
oosrsq.ols_ptv = computeoosrsq(lhs_rsq(params.ntraining+1:end,1),ols_ptv_predy,simple_avg); 
oosrsq.wls = computeoosrsq(lhs_rsq(params.ntraining+1:end,1),wls_predy,simple_avg_wls); 
oosrsq.wls_ct = computeoosrsq(lhs_rsq(params.ntraining+1:end,1),wls_ct_predy,simple_avg_wls); 
oosrsq.wls_ptv = computeoosrsq(lhs_rsq(params.ntraining+1:end,1),wls_ptv_predy,simple_avg_wls); 

ols_rsq_sim = computeoosrsq(lhs_rsq_sim(params.ntraining+1:end,:),ols_predy_sim,simple_avg_sim); 
oosrsq.ols_pvalue = mean( oosrsq.ols < ols_rsq_sim);

ols_ct_rsq_sim = computeoosrsq(lhs_rsq_sim(params.ntraining+1:end,:),ols_ct_predy_sim,simple_avg_sim); 
oosrsq.ols_ct_pvalue = mean( oosrsq.ols_ct < ols_ct_rsq_sim);

ols_ptv_rsq_sim = computeoosrsq(lhs_rsq_sim(params.ntraining+1:end,:),ols_ptv_predy_sim,simple_avg_sim); 
oosrsq.ols_ptv_pvalue = mean( oosrsq.ols_ptv < ols_ptv_rsq_sim);

wls_rsq_sim = computeoosrsq(lhs_rsq_sim(params.ntraining+1:end,:),wls_predy_sim,simple_avg_sim_wls); 
oosrsq.wls_pvalue = mean( oosrsq.wls < wls_rsq_sim);

wls_ct_rsq_sim = computeoosrsq(lhs_rsq_sim(params.ntraining+1:end,:),wls_ct_predy_sim,simple_avg_sim_wls); 
oosrsq.wls_ct_pvalue = mean( oosrsq.wls_ct < wls_ct_rsq_sim);

wls_ptv_rsq_sim = computeoosrsq(lhs_rsq_sim(params.ntraining+1:end,:),wls_ptv_predy_sim,simple_avg_sim_wls); 
oosrsq.wls_ptv_pvalue = mean( oosrsq.wls_ptv < wls_ptv_rsq_sim);

oosrsq.diff_pvalue = mean( (oosrsq.wls - oosrsq.ols) < (wls_rsq_sim - ols_rsq_sim) ); 
oosrsq.diff_ct_pvalue = mean( (oosrsq.wls_ct - oosrsq.ols_ct) < (wls_ct_rsq_sim - ols_ct_rsq_sim) ); 
oosrsq.diff_ptv_pvalue = mean( (oosrsq.wls_ptv - oosrsq.ols_ptv) < (wls_ptv_rsq_sim - ols_ptv_rsq_sim) ); 

% output coeff
oosrsq.ols_coeff = ols_coeff; 
oosrsq.ols_ptv_coeff = ols_ptv_coeff; 
oosrsq.wls_coeff = wls_coeff; 
oosrsq.wls_ptv_coeff = wls_ptv_coeff; 

% output rsq sims
oosrsq.ols_sim = ols_rsq_sim; 
oosrsq.wls_sim = wls_rsq_sim; 

% output cumulative error improvement relative to simple avg
oosrsq.ols_cumimprovement = cumsum( ( lhs_rsq(params.ntraining+1:end,1) - simple_avg ).^2 - ...
    ( lhs_rsq(params.ntraining+1:end,1) - ols_predy ).^2 ); 
oosrsq.wls_cumimprovement = cumsum( ( lhs_rsq(params.ntraining+1:end,1) - simple_avg ).^2 - ...
    ( lhs_rsq(params.ntraining+1:end,1) - wls_predy ).^2 );

end

function rsq = computeoosrsq(r,predr,benchmark_predr)   
%     num_r = size(r,2);
    rsq = 1 - mean((r - predr).^2,1) ./ mean((r - benchmark_predr).^2,1); 
end

function rhs = makerhs(x,nfutperiods)
    rollx = conv(x,ones(1,nfutperiods),'valid'); 
    rollx = rollx(~isnan(rollx),1); 
%     rollx = rollx*var(x)/var(rollx); % DON'T NEED TO DO THIS HERE BECAUSE
%     WE SCALE AT A DIFFERENT POINT

    rhs = [ ones(length(x)-nfutperiods,1) rollx(1:end-1,1) ];  
end