function sim_r = sim_returns(r_data,truevar,params)
% function simr = sim_returns(r_data,truevar,params)
% requires: params.nsim
% optional (if params.use_ferson=true) params.rho_er, params.sig_er
%          params.homosked 

T = length(r_data);

% Estimate return residuals for simulations
if( ~isfield(params,'homosked') || ~params.homosked )    
    % heteroskedastic simulations
   
    % this is the mu_r that produces data_eps with mean zero
    mu_r = mean(r_data ./ sqrt(truevar)) / mean(1 ./ sqrt(truevar));
    data_eps = (r_data - mu_r) ./ sqrt(truevar); 
    % standardize the data_eps so they also have stdev of 1
    data_eps = data_eps / std(data_eps); 
    
    % this is the matrix of simulated eps
    sim_eps_mat = data_eps(randi(T,T,params.nsim),1); 
    sim_eps_mat = reshape(sim_eps_mat,T,params.nsim);
    
    % simulated returns in the heteroskedastic Wild Bootstrap:
    sim_r = mu_r + (sqrt(truevar)*ones(1,params.nsim)).*sim_eps_mat ;   
else
    % homoskedastic simulatoins
    mu_r = mean(r_data); 
    data_eps = r_data - mu_r; 
    
    sim_eps_mat = data_eps(randi(T,T,params.nsim),1); 
    sim_eps_mat = reshape(sim_eps_mat,T,params.nsim);

    sim_r = mu_r + sim_eps_mat; 
end

% add in slow-moving time varying mean (Ferson et al 2003)?
if( isfield(params,'use_ferson') && params.use_ferson )
    % generate time-varying mean process
    % mu_r_t = mu_r + rho_er*(mu_r_{t-1}) + gamma_t
    % gamma_t is N(0,sigma), where sigma is chosen such that std(mu_r_t) =
    % params.std_er
    
    sim_gamma = randn(T,params.nsim)*( params.sig_er * sqrt(1-params.rho_er^2) ); 
    
    % simulate the ar1 we need
    csgammas = cumsum(sim_gamma(2:end,:) ./ ( params.rho_er .^(-T/2:T/2-2)'*ones(1,params.nsim)) ); 
    init_er = mu_r + params.sig_er*randn(1,params.nsim) ; 
    sim_er = [ init_er; mu_r + params.rho_er.^((1:T-1)')*(init_er-mu_r) + ...
        csgammas.*( params.rho_er.^((-T/2:T/2-2)')*ones(1,params.nsim) )]; 
    
    % Add in the time-varying mean to our current simulation
    sim_r = sim_r - mu_r + sim_er; 
end