function writetable_tj(filename,table,coeffrows,serows,params)
% function writetable_tj(filename,table,coeffrows,serows,params)
% Requires coefficients in rows coeffrows and stderrors/tstats in rows
% serows
% 
% params contains:
% numdigits = number of post-decimal digits to use,
% secondrow = 'tstats' or 'stderrors'/'ses' or 'pvalues'/'pvals'

fID = fopen(filename,'w'); 

[M, N] = size(table); 

for row = 1:M
    for col = 1:N
        % Is it a -99? if so, this should be a -
        if( table(row,col) == -99 )
            fprintf(fID,'-'); 
        % Is it NaN? If so, leave blank
        elseif( isnan(table(row,col)) )
            fprintf(fID,''); 
        else
        % Are we in a coeff row?
        if (~isempty(find(coeffrows==row,1)))
            serow = serows(coeffrows==row); 
            
            % If so, check for significance
            if( strcmp(params.secondrow,'stderrors') || strcmp(params.secondrow,'ses') )
                pval = 2*normcdf(-abs(table(row,col))/table(serow,col)); 
            elseif( strcmp(params.secondrow,'tstats'))
                pval = 2*normcdf(-abs(table(serow,col))); 
            elseif( strcmp(params.secondrow,'pvalues') || strcmp(params.secondrow,'pvals') )
                pval = table(serow,col); 
            end
            
            if( pval < 0.01 )
                sigstr = '***'; 
            elseif( pval < 0.05)
                sigstr = '**'; 
            elseif( pval < 0.1)
                sigstr = '*';
            else
                sigstr = ''; 
            end
            if( isfield(params,'as_percent') && params.as_percent )
                fprintf(fID,['%.' num2str(params.numdigits) 'f%%' sigstr ],100*table(row,col)); 
            else
                fprintf(fID,['%.' num2str(params.numdigits) 'f' sigstr ],table(row,col)); 
            end
        % Are we in a SE row (must follow coeff row)
        elseif (~isempty(find(serows==row,1)))
            fprintf(fID,['"=""(%.' num2str(params.numdigits) 'f)"""' ],table(row,col)); 
        % Otherwise, it's something like R^2 or N, just print it
        else
            fprintf(fID,['%.' num2str(params.numdigits) 'f' ],table(row,col)); 
        end
        
        end % end of if( table(row,col) == -99 ) 
        % print a comma if we're not at end of row
        if (col < N)
            fprintf(fID,','); 
        end
        
    end
    % print a new line if we're not done
    if (row < M)
        fprintf(fID,'\n'); 
    end
end

fclose(fID); 